/*
 * This script is run when a skin is selected.
 * The script produces the user interface, and can be edited to add or change user interface options.
 * Author David Ekholm and Anders Callertun
 */
 
import javax.swing.*;
import se.datadosen.component.*;
import net.jalbum.component.*;
import se.datadosen.jalbum.*;
import se.datadosen.util.Item;
import se.datadosen.util.IO;


def fileSelected(ActionEvent ae, JTextField toUpdate) {
	if (window.projectLoadedProperty.get()) {
		JFileSelector selector = (JFileSelector)ae.getSource();
		File sel = selector.getSelectedFile();
		toUpdate.setText(sel.getName());
		File dir = rootFolder.getFile();
		File res = new File(dir, "res");
		res.mkdir();
		IO.copyFile(sel, res);
	}
}

// Build the user interface
ControlPanel ui = new ControlPanel() {
	 ControlPanel generalPanel = new ControlPanel() {
		JTextField separatorImage = new JTextField(20);
		 JComboBox fontFamily = new JComboBox(new Object[] {
			new Item("Arial, \"Helvetica Neue\", Helvetica, sans-serif", "Arial"),
			new Item("\"Arial Rounded MT Bold\", \"Helvetica Rounded\", Arial, sans-serif;", "Arial Rounded"),
			new Item("Baskerville, \"Baskerville Old Face\", \"Hoefler Text\", Garamond, \"Times New Roman\", serif", "Baskerville"),
			new Item("Calibri, Candara, Segoe, \"Segoe UI\", Optima, Arial, sans-serif", "Calibri"),
			new Item("Cambria, Georgia, Times, \"Times New Roman\", serif", "Cambria"),
			new Item("Candara, Calibri, Segoe, \"Segoe UI\", Optima, Arial, sans-serif", "Candara"),
			new Item("\"Century Gothic\", \"Apple Gothic\", \"Goudy Old Style\", sans-serif", "Century Gothic"),
			new Item("\"Comic Sans MS\", cursive", "Comic Sans"),
			new Item("Consolas, \"Lucida Console\", Monaco, monospace", "Consolas"),
			new Item("Constantia, Palatino, \"Palatino Linotype\", \"Palatino LT STD\", Georgia, serif", "Constantia"),
			new Item("\"Copperplate Light\", \"Copperplate Gothic Light\", serif", "Copperplate Light"),
			new Item("\"Courier New\", Courier, monospace", "Courier New"),
			new Item("\"Franklin Gothic Medium\", \"Arial Narrow Bold\", Arial, sans-serif", "Franklin Gothic"),
			new Item("Futura, \"Century Gothic\", AppleGothic, sans-serif", "Futura"),
			new Item("Garamond, \"Hoefler Text\", Times New Roman, Times, serif", "Garamond"),
			new Item("Geneva, \"Lucida Sans\", \"Lucida Grande\", \"Lucida Sans Unicode\", Verdana, sans-serif", "Geneva"),
			new Item("Georgia, Palatino, \"Palatino Linotype\", Times, \"Times New Roman\", serif", "Georgia"),
			new Item("\"Gill Sans\", \"Gill Sans MT\", Calibri, \"Trebuchet MS\", sans-serif", "Gill Sans"),
			new Item("\"Goudy Old Style\", Garamond, \"Big Caslon\", \"Times New Roman\", serif", "Goudy Old Style"),
			new Item("\"Helvetica Neue\", Helvetica, Arial, sans-serif", "Helvetica Neue"),
			new Item("\"Hoefler Text\", Constantia, Palatino, \"Palatino Linotype\", \"Book Antiqua\", Georgia, serif", "Hoefler Text"),
			new Item("Impact, Haettenschweiler, \"Arial Narrow Bold\", sans-serif", "Impact"),
			new Item("\"Lucida Sans\", \"Lucida Grande\", \"Lucida Sans Unicode\", Verdana, sans-serif", "Lucida Sans"),
			new Item("\"Lucida Bright\", Georgia, serif", "Lucida Bright"),
			new Item("Palatino, \"Palatino Linotype\", \"Book Antiqua\", Georgia, serif", "Palatino"),
			new Item("Segoe, \"Segoe UI\", Tahoma, Geneva, \"Nimbus Sans L\", sans-serif", "Segoe"),
			new Item("Tahoma, Geneva, Verdana, sans-serif", "Tahoma"),
			new Item("Times, \"Times New Roman\", Georgia, serif", "Times"),
			new Item("\"Trebuchet MS\", \"Lucida Sans Unicode\", \"Lucida Grande\", \"Lucida Sans\", Tahoma, sans-serif", "Trebuchet MS"),
			new Item("Verdana, Geneva, Tahoma, sans-serif", "Verdana") 
		});
		JComboBox headlineFont = new JComboBox(new Object[] {
			new Item("", "[ Same as base font ]"),
			"Abril Fatface",
			"Alex Brush",
			"Alegreya Sans SC",
			"Amaranth",
			"Amatic SC",
			"Anton",
			"Arapey",
			"Cantata One",
			"Cinzel Decorative",
			"Cookie",
			"Dancing Script",
			"Dosis",
			"Dynalight",
			"Economica",
			"Emilys Candy",
			"Euphoria Script",
			"Fauna One",
			"Fjalla One",
			"Fredericka the Great",
			"Germania One",
			"Geo",
			"Gilda Display",
			"Grand Hotel",
			"Great Vibes",
			"Gruppo",
			"IM Fell English",
			"Indie Flower",
			"Italiana",
			"Josefin Sans",
			"Josefin Slab",
			"Julius Sans One",
			"La Belle Aurore",
			"Libre Baskerville",
			"Lobster",
			"Lobster Two",
			"Lora",
			"Loved by the King",
			"Maiden Orange",
			"Marvel",
			"Medula One",
			"Mountains of Christmas",
			"Noticia Text",
			"Old Standard TT",
			"Open Sans Condensed:300",
			"Oswald",
			"Oranienbaum", 
			"Philosopher",
			"Poiret One",
			"Prata",
			"Princess Sofia",
			"PT Mono",
			"PT Sans Narrow",
			"Raleway",
			"Roboto Condensed",
			"Rochester",
			"Scope One",
			"Shadows Into Light Two",
			"Six Caps",					
			"Sorts Mill Goudy",
			"Special Elite",
			"Squada One",
			"Strait",
			"Tulpen One",
			"Ubuntu Condensed",
			"Unica One",
			"Vidaloka",
			"Yanone Kaffeesatz:300",
			"Yanone Kaffeesatz"
		});
		JCheckBox headlineUpperCase = new JCheckBox(texts.getString("ui.allCaps"));
		
		{
			JFileSelector separatorSelector = new JFileSelector(texts.getString("ui.separatorImageTitle"));
			separatorSelector.addActionListener(ae -> fileSelected(ae, separatorImage));
			fontFamily.setSelectedIndex(23);
			headlineFont.setEditable(true);
			headlineFont.setSelectedIndex(30);
			
			add("br label", new JLabel(texts.getString("ui.separatorImage")));
			add("tab", separatorImage);
			add(separatorSelector);
			add("br label", new JLabel(texts.getString("ui.fontFamily")));
			add("tab", fontFamily);
			add("br label", new JLabel(texts.getString("ui.headlineFont")));
			add("tab", headlineFont);
			add("tab", headlineUpperCase);
		}
	};

	ControlPanel backgroundPanel = new ControlPanel() {
		JCheckBox fixedThemeImage = new JCheckBox((texts.getString("ui.fixedThemeImage")));
		JCheckBox stretchedThemeImage = new JCheckBox((texts.getString("ui.stretchedThemeImage")));
		JColorSelector backgroundColor = new JAlphaColorSelector(texts.getString("ui.backgroundColor"), new JTextField(7));
		{
			add("br", fixedThemeImage);
			add("br", stretchedThemeImage);
			add("br label", new JLabel(texts.getString("ui.backgroundColor")));
			add("tab", backgroundColor);
			add(backgroundColor.getTextComponent());			
		}		
	};
	
	ControlPanel introPanel = new ControlPanel() {
		JColorSelector introTextColor = new JAlphaColorSelector(texts.getString("ui.introPanelTextColorTitle"), new JTextField(7));
		JTextField centerLogo = new JTextField(20);
		JTextField cornerLogo = new JTextField(20);
		JCheckBox showToItems = new JCheckBox(texts.getString("ui.showToItemsButton"));
		{
			JFileSelector centerLogoSelector = new JFileSelector(texts.getString("ui.centerLogoTitle"));
			centerLogoSelector.addActionListener(ae -> fileSelected(ae, centerLogo));
			JFileSelector cornerLogoSelector = new JFileSelector(texts.getString("ui.cornerLogoTitle"));
			cornerLogoSelector.addActionListener(ae -> fileSelected(ae, cornerLogo));
			
			add("br label", new JLabel(texts.getString("ui.centerLogo")));
			add("tab", centerLogo);
			add(centerLogoSelector);
			add("br label", new JLabel(texts.getString("ui.cornerLogo")));
			add("tab", cornerLogo);
			add(cornerLogoSelector);
			add("br label", new JLabel(texts.getString("ui.textColor")));
			add(introTextColor);
			add("tab", introTextColor.getTextComponent());
			add("br", showToItems);
		}
	};
	
	ControlPanel itemsPanel = new ControlPanel() {
		JColorSelector itemHeaderColor = new JAlphaColorSelector(texts.getString("ui.itemsHeaderColorTitle"), new JTextField(7));
		JColorSelector itemTextColor = new JAlphaColorSelector(texts.getString("ui.itemsTextColorTitle"), new JTextField(7));
		JColorSelector itemBackgroundColor = new JAlphaColorSelector(texts.getString("ui.itemsBackgroundColorTitle"), new JTextField(7));
		JCheckBox showToTop = new JCheckBox(texts.getString("ui.showToTopButton"));
		JCheckBox reverseOrder = new JCheckBox(texts.getString("ui.reverseOrder"));
		
		{
			add("br label", new JLabel(texts.getString("ui.headerColor")));
			add("tab", itemHeaderColor);
			add(itemHeaderColor.getTextComponent());
			add("br label", new JLabel(texts.getString("ui.textColor")));
			add("tab", itemTextColor);			
			add(itemTextColor.getTextComponent());
			add("br label", new JLabel(texts.getString("ui.backgroundColor")));
			add("tab", itemBackgroundColor);
			add(itemBackgroundColor.getTextComponent());
			add("br", showToTop);
			add("br", reverseOrder);
		}
	};
	
	ControlPanel footerPanel = new ControlPanel() {
		JTextArea footerText = new JTextArea(4,40);
		{
			add("br hfill", new JScrollPane(footerText));
		}
	};

	ControlPanel advancedPanel = new ControlPanel() {
		JTextArea customCSS = new JTextArea(8,40);
		{
			add("br", new JLabel(texts.getString("ui.customCSS")));
			add("br hfill", new JScrollPane(customCSS));
		}
	};
	
	// Tabbed pane
	JTabbedPane tabs = new JTabbedPane() {
		{
			add(generalPanel, texts.getString("ui.general"));
			add(backgroundPanel, texts.getString("ui.background"));
			add(introPanel, texts.getString("ui.introPanel"));
			add(itemsPanel, texts.getString("ui.items"));
			add(footerPanel,texts.getString("ui.footerText"));
			add(advancedPanel,texts.getString("ui.advanced"));
		}
	};
	{
		add("vtop hfill vfill", tabs);
		putClientProperty("helpPage", "http://jalbum.net/help/Skin/Zigzag");
	}
};

window.setSkinUI(ui);
